import unittest
import bencher as bch
from bencher.example.meta.example_meta import BenchableObject


class TestBenchMeta(unittest.TestCase):
    def test_repeats_equal(self):
        # bench = bch.Bench("meta", BenchMeta())

        bench = bch.Bench("bench", BenchableObject())

        res1 = bench.plot_sweep(
            "repeats", input_vars=[BenchableObject.param.float1], run_cfg=bch.BenchRunCfg(repeats=1)
        )

        res1_eq = bench.plot_sweep(
            "repeats", input_vars=[BenchableObject.param.float1], run_cfg=bch.BenchRunCfg(repeats=1)
        )

        res2 = bench.plot_sweep(
            "repeats", input_vars=[BenchableObject.param.float1], run_cfg=bch.BenchRunCfg(repeats=2)
        )

        self.assertTrue(
            res1.to_dataset(bch.ReduceType.NONE).identical(res1_eq.to_dataset(bch.ReduceType.NONE)),
            "created with identical settings so should be equal",
        )

        self.assertFalse(
            res1.to_dataset(bch.ReduceType.NONE).identical(res2.to_dataset(bch.ReduceType.NONE)),
            "different number of repeats so should not be equal",
        )

        res1_ds = res1.to_dataset(bch.ReduceType.SQUEEZE)
        res2_ds = res2.to_dataset(bch.ReduceType.REDUCE)

        self.assertFalse(
            res1_ds.identical(res2_ds), "should not be equal because of std_dev column"
        )

        res2_ds = res2_ds.drop_vars("distance_std")
        res2_ds = res2_ds.drop_vars("sample_noise_std")

        print(res1_ds)
        print(res2_ds)
        print(res1_ds["distance"].attrs)
        print(res2_ds["distance"].attrs)

        self.assertTrue(
            res1_ds.equals(res2_ds), "should be equal because of removed std_dev column"
        )

        self.assertTrue(
            res1_ds.identical(res2_ds), "should be equal because of removed std_dev column"
        )



import unittest
import bencher as bch
import numpy as np
from hypothesis import given, strategies as st


class TestComposableContainerVideo(unittest.TestCase):
    def small_img(self, size=None):
        if size is None:
            size = (2, 1)
        return np.ones((size[0], size[1], 3))
        # return np.array(
        #     [[[1, 1, 1]], [[0, 0, 0]]]
        # )  # represents an image of 1x2 (keep it small for speed)

    def small_video(self, num_frames: int = 2, render_cfg=None, size=None):
        img = self.small_img(size=size)
        vid = bch.ComposableContainerVideo()
        for _ in range(num_frames):
            vid.append(img)
        return vid.render(render_cfg)

    # @given(frames=st.sampled_from([1, 2, 10, 100]))
    # def test_duration_default(self, frames):
    #     ccv = bch.ComposableContainerVideo()
    #     duration, frame_duration = ccv.calculate_duration(frames, bch.RenderCfg())
    #     self.assertEqual(duration, 10)
    #     self.assertEqual(frame_duration, 10 / frames)

    @given(frames=st.sampled_from([1, 2, 10, 100]), duration=st.sampled_from([0.1, 1, 10, 100]))
    def test_set_duration(self, frames, duration):
        ccv = bch.ComposableContainerVideo()
        duration, frame_duration = ccv.calculate_duration(frames, bch.RenderCfg(duration=duration))
        self.assertEqual(duration, duration)
        self.assertEqual(frame_duration, duration / frames)

    @given(frames=st.sampled_from([1, 2, 10, 100]), duration=st.sampled_from([0.1, 1, 10, 100]))
    def test_set_duration1(self, frames, duration):
        ccv = bch.ComposableContainerVideo()
        min_frame_duration = 1 / 30
        max_frame_duration = 2.0
        duration, frame_duration = ccv.calculate_duration(
            frames,
            bch.RenderCfg(
                duration=duration,
                min_frame_duration=1 / 30,
                max_frame_duration=2,
                duration_target=True,
            ),
        )
        self.assertEqual(duration, duration)
        self.assertEqual(frame_duration, duration / frames)
        self.assertLessEqual(frame_duration, max_frame_duration)
        self.assertGreaterEqual(frame_duration, min_frame_duration)

    def test_img_right(self):
        res = self.small_video(1, bch.RenderCfg(duration=0.1, compose_method=bch.ComposeType.right))
        self.assertEqual(res.size, (1, 2))
        self.assertEqual(res.duration, 0.1)

    def test_img_down(self):
        res = self.small_video(1, bch.RenderCfg(duration=0.1, compose_method=bch.ComposeType.down))
        self.assertEqual(res.size, (1, 2))
        self.assertEqual(res.duration, 0.1)

    def test_img_sequence(self):
        res = self.small_video(
            1, bch.RenderCfg(duration=0.1, compose_method=bch.ComposeType.sequence)
        )
        self.assertEqual(res.size, (1, 2))
        self.assertEqual(res.duration, 0.1)

    def test_img_right_x2(self):
        res = self.small_video(2, bch.RenderCfg(compose_method=bch.ComposeType.right))
        self.assertEqual(res.size, (2, 2))
        self.assertEqual(res.duration, 2)

    def test_img_down_x2(self):
        res = self.small_video(2, bch.RenderCfg(compose_method=bch.ComposeType.down))
        self.assertEqual(res.size, (1, 4))
        self.assertEqual(res.duration, 2)

    def test_img_seq_x2(self):
        res = self.small_video(2, bch.RenderCfg(compose_method=bch.ComposeType.sequence))
        self.assertEqual(res.size, (1, 2))
        self.assertEqual(res.duration, 4)

    def test_video_seq(self):
        img = self.small_img()
        vid1 = bch.ComposableContainerVideo()
        vid1.append(img)
        vid1.append(img)

        res = vid1.deep().render(
            bch.RenderCfg(duration=0.1, compose_method=bch.ComposeType.sequence)
        )

        self.assertEqual(res.size, (1, 2))
        self.assertEqual(res.duration, 0.1)  # two frames

        render_cfg = bch.RenderCfg(
            duration=10.0,
            duration_target=True,
            min_frame_duration=1.0 / 30,
            max_frame_duration=1.0,
        )

        res = vid1.deep().render(render_cfg)

        self.assertEqual(res.size, (1, 2))
        self.assertEqual(res.duration, 2.0)  # 2 frames of minimum frame time 1.0

        vid1.append(img)

        res = vid1.render(render_cfg)
        self.assertEqual(res.size, (1, 2))
        self.assertEqual(res.duration, 3.0)  # 3 frames of minimum frame time 1.0

    def test_concat_equal_video_len(self):
        render_cfg = bch.RenderCfg(duration_target=True)
        res1 = self.small_video(num_frames=2, render_cfg=render_cfg)
        res2 = self.small_video(num_frames=2, render_cfg=render_cfg)

        self.assertEqual(res1.size, (1, 2))
        self.assertEqual(res1.duration, 4.0)  # 2 frames of minimum frame time 2.0

        self.assertEqual(res2.size, (1, 2))
        self.assertEqual(res2.duration, 4.0)  # 1 frames of minimum frame time 2.0

        vid = bch.ComposableContainerVideo()

        vid.append(res1)
        vid.append(res2)

        res = vid.deep().render(bch.RenderCfg(bch.ComposeType.right))
        self.assertEqual(res.size, (2, 2))
        self.assertEqual(res.duration, 4.0)  # the duration of the longer clip

        res = vid.deep().render(bch.RenderCfg(bch.ComposeType.down))
        self.assertEqual(res.size, (1, 4))
        self.assertEqual(res.duration, 4.0)  # the duration of the longer clip

        res = vid.deep().render(bch.RenderCfg(bch.ComposeType.sequence))
        self.assertEqual(res.size, (1, 2))
        self.assertEqual(res.duration, 8.0)  # the duration of both clips

    def test_concat_unequal_video_len(self):
        render_cfg = bch.RenderCfg(duration_target=True)
        res1 = self.small_video(num_frames=2, render_cfg=render_cfg)
        self.assertEqual(res1.size, (1, 2))
        self.assertEqual(res1.duration, 4.0)  # 3 frames of minimum frame time 1.0

        res2 = self.small_video(num_frames=3, render_cfg=render_cfg)
        self.assertEqual(res2.size, (1, 2))
        self.assertEqual(res2.duration, 6.0)  # 3 frames of minimum frame time 1.0

        vid = bch.ComposableContainerVideo()

        vid.append(res1)
        vid.append(res2)

        res = vid.deep().render(bch.RenderCfg(bch.ComposeType.right))
        self.assertEqual(res.size, (2, 2))
        self.assertEqual(res.duration, 6.0)  # the duration of the longer clip

        res = vid.deep().render(bch.RenderCfg(bch.ComposeType.down))
        self.assertEqual(res.size, (1, 4))
        self.assertEqual(res.duration, 6.0)  # the duration of the longer clip

        res = vid.deep().render(bch.RenderCfg(bch.ComposeType.sequence))
        self.assertEqual(res.size, (1, 2))
        self.assertEqual(res.duration, 10.0)  # the duration of both clips

        res_label = vid.deep().render(bch.RenderCfg(bch.ComposeType.sequence))
        self.assertEqual(res.duration, res_label.duration)  # the duration of both clips

    def test_bad_filetype(self):
        vid = bch.ComposableContainerVideo()
        with self.assertRaises(RuntimeWarning):
            vid.append("bad.badextension")

    def test_simple_image_length(self):
        ccv = bch.ComposableContainerVideo()
        ccv.append(self.small_img())
        ccv.append(self.small_img())

        res = ccv.render()
        self.assertEqual(res.duration, 4.0)  # limited by maximum frame time

        ccv = bch.ComposableContainerVideo()
        for _ in range(20):
            ccv.append(self.small_img())

        self.assertEqual(ccv.render().duration, 10.0)  # limited by target video duration

        ccv = bch.ComposableContainerVideo()
        for _ in range(200):
            ccv.append(self.small_img())

        # still limited by target video duration
        self.assertAlmostEqual(ccv.render().duration, 10.0)

    def test_composite_image_length(self):
        ccv = bch.ComposableContainerVideo()

        for _ in range(2):
            sub_img = bch.ComposableContainerVideo()
            sub_img.append(self.small_img())
            sub_img.append(self.small_img())
            sub_img.append(self.small_img())
            ccv.append(sub_img.render(compose_method=bch.ComposeType.right))

        # should be 4 because there are two sequential frames
        self.assertEqual(ccv.render().duration, 4.0)

        ccv = bch.ComposableContainerVideo()

        for _ in range(5):
            sub_img = bch.ComposableContainerVideo()
            sub_img.append(self.small_img())
            sub_img.append(self.small_img())
            sub_img.append(self.small_img())
            ccv.append(sub_img.render(compose_method=bch.ComposeType.right))

        # should be 10 because of the default target length
        self.assertEqual(ccv.render().duration, 10.0)

    def test_video_lengths(self):
        ccv = bch.ComposableContainerVideo()
        ccv.append(self.small_video())
        ccv.append(self.small_video())

        res = ccv.render()
        self.assertEqual(res.duration, 8.0)  # no frame limits, just concat videos

        ccv = bch.ComposableContainerVideo()
        for _ in range(20):
            ccv.append(self.small_video())

        self.assertEqual(ccv.render().duration, 80.0)  # concatted vid time

        ccv = bch.ComposableContainerVideo()
        for _ in range(200):
            ccv.append(self.small_video())

        # still concatted vid time
        self.assertAlmostEqual(ccv.render().duration, 800.0)


if __name__ == "__main__":
    tst = TestComposableContainerVideo()

    tst.test_img_right()

    unittest.main()



import unittest
import bencher as bch
import numpy as np

from bencher.example.meta.example_meta import BenchableObject


class TestBench(bch.ParametrizedSweep):
    float_var = bch.FloatSweep(default=0, bounds=[0, 4])
    cat_var = bch.StringSweep(["a", "b", "c", "d", "e"])
    result = bch.ResultVar()

    def __call__(self, **kwargs):
        self.result = 1
        return super().__call__()


class TestBenchResultBase(unittest.TestCase):
    def test_to_dataset(self):
        bench = BenchableObject().to_bench()

        res_repeat1 = bench.plot_sweep(
            "sweep1repeat",
            input_vars=[BenchableObject.param.float1],
            result_vars=[BenchableObject.param.distance, BenchableObject.param.sample_noise],
            run_cfg=bch.BenchRunCfg(repeats=1),
            plot_callbacks=False,
        )

        res_repeat2 = bench.plot_sweep(
            "sweep2repeat",
            input_vars=[BenchableObject.param.float1],
            result_vars=[BenchableObject.param.distance, BenchableObject.param.sample_noise],
            run_cfg=bch.BenchRunCfg(repeats=2),
            plot_callbacks=False,
        )

        # print(res_repeat1.to_dataset())
        # print(res_repeat1.to_hv_dataset().data)
        # print(res_repeat1.to_hv_dataset_old().data)

        # print(res_repeat1.to_dataset()["distance"].attrs)

        # print(res_repeat2.to_dataset())
        # print(res_repeat2.to_hv_dataset())
        # print(res_repeat2.to_dataset()["distance"].attrs)

        self.assertEqual(
            res_repeat1.to_dataset()["distance"].attrs, res_repeat2.to_dataset()["distance"].attrs
        )

        # bm.__call__(float_vars=1, sample_with_repeats=1)

    def test_select_level(self):
        bench = TestBench().to_bench()

        res = bench.plot_sweep(
            input_vars=["float_var", "cat_var"],
            run_cfg=bch.BenchRunCfg(level=4),
            plot_callbacks=False,
        )

        def asserts(ds, expected_float, expected_cat):
            np.testing.assert_array_equal(
                ds.coords["float_var"].to_numpy(), np.array(expected_float, dtype=float)
            )
            np.testing.assert_array_equal(ds.coords["cat_var"].to_numpy(), np.array(expected_cat))

        ds_raw = res.to_dataset()
        asserts(ds_raw, [0, 1, 2, 3, 4], ["a", "b", "c", "d", "e"])

        ds_filtered_all = res.select_level(ds_raw, 2)
        asserts(ds_filtered_all, [0, 4], ["a", "e"])

        ds_filtered_types = res.select_level(ds_raw, 2, float)
        asserts(ds_filtered_types, [0, 4], ["a", "b", "c", "d", "e"])

        ds_filtered_names = res.select_level(ds_raw, 2, exclude_names="cat_var")
        asserts(ds_filtered_names, [0, 4], ["a", "b", "c", "d", "e"])



import bencher as bch
import unittest
from bencher.utils import (
    get_nearest_coords,
    capitalise_words,
    int_to_col,
    get_nearest_coords1D,
    callable_name,
    lerp,
    listify,
    tabs_in_markdown,
    mult_tuple,
)
from functools import partial
import xarray as xr


class ExampleClass(bch.ParametrizedSweep):
    iv1 = bch.FloatSweep()
    rv1 = bch.ResultVar()


class TestBencherUtils(unittest.TestCase):
    def test_get_inputs(self) -> None:
        ex_instance = ExampleClass()
        inputs = ex_instance.get_inputs_only()

        print(inputs)

        self.assertEqual(len(inputs), 1)
        self.assertEqual(inputs[0].name, "iv1")

    def test_get_results(self) -> None:
        ex_instance = ExampleClass()
        results = ex_instance.get_results_only()

        self.assertEqual(len(results), 1)
        self.assertEqual(results[0].name, "rv1")

    def test_get_inputs_and_results(self) -> None:
        ex_instance = ExampleClass()
        inputs, results = ex_instance.get_input_and_results()

        self.assertEqual(len(inputs), 1)
        self.assertEqual(len(results), 1)
        self.assertEqual(inputs["iv1"].name, "iv1")
        self.assertEqual(results["rv1"].name, "rv1")

    def test_get_results_values_as_dict(self) -> None:
        ex_instance = ExampleClass()

        ex_instance.rv1 = 3

        res = ex_instance.get_results_values_as_dict()

        self.assertEqual(res["rv1"], 3)

        # Tests that a named tuple with fields of different data types is created successfully

    def test_different_datatypes_namedtuple(self):
        result = bch.make_namedtuple("Test", field1=1, field2="value2", field3=True)
        self.assertEqual(result.field1, 1)
        self.assertEqual(result.field2, "value2")
        self.assertEqual(result.field3, True)

        # Tests that the function returns an empty tuple when an empty dictionary is passed as input

    def test_edge_case_empty_dictionary(self) -> None:
        input_dict = {}
        expected_output = ()
        self.assertEqual(bch.hmap_canonical_input(input_dict), expected_output)

    def test_dictionary_order(self) -> None:
        dic1 = {"x": 1, "y": 2}
        dic2 = {"y": 2, "x": 1}

        self.assertEqual(
            bch.hmap_canonical_input(dic1),
            bch.hmap_canonical_input(dic2),
        )

    def test_mult_tuple(self) -> None:
        self.assertTupleEqual(mult_tuple((1, 2, 3), 2), (2, 4, 6))

    # Tests that the function returns the nearest coordinate name value pair for a dataset containing multiple coordinates
    def test_multiple_coordinates(self):
        ds = xr.Dataset({"x": [1, 2, 3], "y": [4, 5, 6]})
        result = get_nearest_coords(ds, x=2.5, y=5.5)
        self.assertEqual(result, {"x": 3, "y": 6})

    def test_capitalise_words(self):
        self.assertEqual("Camel Case", capitalise_words("camel case"))

    def test_int_to_col(self):
        self.assertEqual(int_to_col(0), (0.95, 0.475, 0.475))
        self.assertEqual(int_to_col(0, alpha=1), (0.95, 0.475, 0.475, 1))

    def test_nearest_coords(self):
        self.assertEqual(get_nearest_coords1D(1, [0, 1, 2]), 1)
        self.assertEqual(get_nearest_coords1D(100, [0, 1, 2]), 2)
        self.assertEqual(get_nearest_coords1D("b", ["a", "b", "c"]), "b")

    def test_returns_name_of_original_function_for_partial_function(self):
        # Arrange
        def original_function():
            pass

        partial_function = partial(original_function)

        # Act
        result = callable_name(partial_function)

        # Assert
        self.assertEqual(result, "original_function")

    # The function returns the name of a given callable function.
    def test_returns_name_of_callable_function(self) -> None:
        # Arrange
        def my_function():
            pass

        # Act
        result = callable_name(my_function)

        # Assert
        self.assertEqual(result, "my_function")

    def test_callable_name_incorrect(self) -> None:
        self.assertEqual(callable_name("lol"), "lol")

    def test_lerp(self):
        # Given valid input values, the function should return the expected output.
        result = lerp(5, 0, 10, 0, 100)
        self.assertEqual(result, 50)

        # When the input value is equal to the input_low, the function should return output_low.
        result = lerp(0, 0, 10, 0, 100)
        self.assertEqual(result, 0)

        # When the input value is equal to the input_high, the function should return output_high.
        result = lerp(10, 0, 10, 0, 100)
        self.assertEqual(result, 100)

        # When the input value is None, the function should raise a TypeError.
        with self.assertRaises(TypeError):
            lerp(None, 0, 10, 0, 100)

        # When the input_low is None, the function should raise a TypeError.
        with self.assertRaises(TypeError):
            lerp(5, None, 10, 0, 100)

        # When the input_high is None, the function should raise a TypeError.
        with self.assertRaises(TypeError):
            lerp(5, 0, None, 0, 100)

    def test_listify(self):
        obj = "a"
        self.assertEqual([obj], listify(obj))
        self.assertEqual([obj], listify([obj]))
        self.assertEqual([obj], listify((obj)))
        self.assertEqual(None, listify(None))

    def test_converts_single_tab_to_nbsp(self):
        input_str = "This is\ta test"
        expected_output = "This is&nbsp;&nbsp;a test"
        self.assertEqual(tabs_in_markdown(input_str), expected_output)

    def test_converts_multi_tab_to_nbsp(self):
        input_str = "This is\ta test"
        expected_output = "This is&nbsp;&nbsp;&nbsp;&nbsp;a test"
        self.assertEqual(tabs_in_markdown(input_str, 4), expected_output)

    def test_handles_empty_string(self):
        input_str = ""
        expected_output = ""
        self.assertEqual(tabs_in_markdown(input_str), expected_output)



# Generated by CodiumAI
import unittest
from unittest.mock import Mock
from bencher.example.benchmark_data import SimpleBenchClass, SimpleBenchClassFloat
import bencher as bch


class TestBenchRunner(unittest.TestCase):
    # Tests that bch.BenchRunner can be created with default configuration and the import statement in the bch.BenchRunner class is fixed
    def test_benchrunner_default_configuration_fixed(self):
        bench_runner = bch.BenchRunner("bench_runner_test")
        self.assertEqual(bench_runner.run_cfg.use_sample_cache, True)
        self.assertEqual(bench_runner.run_cfg.only_hash_tag, True)
        self.assertEqual(bench_runner.run_cfg.level, 2)
        self.assertEqual(bench_runner.publisher, None)
        self.assertEqual(bench_runner.bench_fns, [])

    # Tests that Benchable functions can be added to bch.BenchRunner instance
    def test_benchrunner_add_benchable_functions(self):
        bench_runner = bch.BenchRunner("bench_runner_test")
        bench_fn1 = Mock()
        bench_fn2 = Mock()
        bench_runner.add_run(bench_fn1)
        bench_runner.add_run(bench_fn2)
        self.assertEqual(len(bench_runner.bench_fns), 2)
        self.assertIn(bench_fn1, bench_runner.bench_fns)
        self.assertIn(bench_fn2, bench_runner.bench_fns)

    def test_benchrunner_handle_empty_list(self):
        bench_runner = bch.BenchRunner("bench_runner_test")
        results = bench_runner.run()
        self.assertEqual(len(results), 0)

    def test_benchrunner_benchable_class(self):
        bench_runner = bch.BenchRunner("bench_runner_test")
        bench_runner.add_bench(SimpleBenchClass())
        results = bench_runner.run(run_cfg=bch.BenchRunCfg(run_tag="1"))

        self.assertEqual(results[0].bench_cfg.run_tag, "1")

    def test_benchrunner_cache(self):
        from datetime import datetime

        run_tag = str(datetime.now())

        bench_runner = bch.BenchRunner(
            "bench_runner_test_cache", run_cfg=bch.BenchRunCfg(run_tag=run_tag)
        )

        bench_class = SimpleBenchClass()
        # bench = bch.Bench("test_bench", bench_class, run_cfg=run_cfg, report=report            )

        def run_bench_class(run_cfg: bch.BenchRunCfg, report: bch.BenchReport) -> bch.BenchCfg:
            bench = bch.Bench("test_bench1_cache", bench_class, run_cfg=run_cfg, report=report)
            bench.plot_sweep("bench_1")
            return bench

        bench_runner.add_run(run_bench_class)

        # run with unique tag and with cache, should not hit cache because unique tag
        results = bench_runner.run()
        self.assertEqual(results[0].sample_cache.worker_wrapper_call_count, 2)
        self.assertEqual(results[0].sample_cache.worker_fn_call_count, 2)
        self.assertEqual(results[0].sample_cache.worker_cache_call_count, 0)
        self.assertEqual(results[0].run_cfg.run_tag, run_tag)

        # run again with the same tag, should hit cache because it was already run
        results = bench_runner.run()
        self.assertEqual(results[0].sample_cache.worker_wrapper_call_count, 2)
        self.assertEqual(results[0].sample_cache.worker_fn_call_count, 2)
        self.assertEqual(results[0].sample_cache.worker_cache_call_count, 0)
        self.assertEqual(results[0].run_cfg.run_tag, run_tag)

        # run with the same tag but set use cache to false, should not hit cache because even tho the tag is the same, use_cache=false
        results = bench_runner.run(use_cache=False)
        self.assertEqual(results[0].sample_cache.worker_wrapper_call_count, 2)
        self.assertEqual(results[0].sample_cache.worker_fn_call_count, 2)
        self.assertEqual(results[0].sample_cache.worker_cache_call_count, 0)
        self.assertEqual(results[0].run_cfg.run_tag, run_tag)

    def test_benchrunner_benchable_class_run_constructor(self):
        bench_runner = bch.BenchRunner("bench_runner_test", run_cfg=bch.BenchRunCfg(run_tag="1"))
        bench_runner.add_bench(SimpleBenchClass())
        results = bench_runner.run()
        self.assertEqual(results[0].bench_cfg.run_tag, "1")

    # def test_benchrunner_level_1(self):
    #     results = bch.BenchRunner("bench_runner_test", AllSweepVars()).run(min_level=1)
    #     self.assertEqual(results[0].result_samples(), 1)

    # def test_benchrunner_level_1_only(self):
    #     results = bch.BenchRunner("bench_runner_test", AllSweepVars()).run(level=1)
    #     self.assertEqual(results[0].result_samples(), 1)

    def test_benchrunner_repeats(self):
        res = bch.Bench(
            "float", SimpleBenchClassFloat(), run_cfg=bch.BenchRunCfg(level=2, repeats=1)
        ).plot_sweep("float")
        self.assertEqual(res.result_samples(), 2)

        res = bch.Bench(
            "float", SimpleBenchClassFloat(), run_cfg=bch.BenchRunCfg(level=2, repeats=5)
        ).plot_sweep("float")
        self.assertEqual(res.result_samples(), 10)

    # def test_benchrunner_cache(self):
    #     res = bch.Bench(
    #         "float", SimpleBenchClassFloat(), run_cfg=bch.BenchRunCfg(level=2, repeats=1)
    #     ).plot_sweep("float")

    #     res = bch.Bench(
    #         "float", SimpleBenchClassFloat(), run_cfg=bch.BenchRunCfg(level=2, repeats=5)
    #     ).plot_sweep("float")
    #     self.assertEqual(res.result_samples(), 10)

    # # Tests that bch.BenchRunner can run Benchable functions with default configuration (fixed)
    # def test_benchrunner_run_default_configuration_fixed(self):

    #     bench_runner = bch.BenchRunner()
    #     bench_fn1 = Mock()
    #     bench_fn2 = Mock()
    #     bench_runner.add_run(bench_fn1)
    #     bench_runner.add_run(bench_fn2)
    #     results = bench_runner.run()

    #     self.assertEqual(len(results), 10)
    #     self.assertEqual(results[0].level, 1)
    #     self.assertEqual(results[1].level, 1)
    #     self.assertEqual(results[2].level, 2)
    #     self.assertEqual(results[3].level, 2)
    #     self.assertEqual(results[4].level, 3)
    #     self.assertEqual(results[5].level, 3)
    #     self.assertEqual(results[6].level, 4)
    #     self.assertEqual(results[7].level, 4)
    #     self.assertEqual(results[8].level, 5)
    #     self.assertEqual(results[9].level, 5)

    # Tests that bch.BenchRunner can run Benchable functions with custom configuration, after fixing the import statements
    # def test_benchrunner_run_custom_configuration_fixed_fixed_import_statements(self):

    #     bench_runner = bch.BenchRunner()
    #     bench_fn1 = Mock()
    #     bench_fn2 = Mock()
    #     bench_runner.add_run(bench_fn1)
    #     bench_runner.add_run(bench_fn2)
    #     run_cfg = bch.BenchRunCfg()
    #     run_cfg.use_sample_cache = False
    #     run_cfg.only_hash_tag = False
    #     run_cfg.level = 3
    #     results = bench_runner.run(run_cfg=run_cfg)
    #     self.assertEqual(len(results), 2)
    #     self.assertEqual(results[0].level, 3)
    #     self.assertEqual(results[1].level, 3)

    # Tests that bch.BenchRunner can publish results of Benchable functions (fixed)
    # def test_benchrunner_publish_results_fixed(self):
    #     class MockBenchable:
    #         def bench(self, run_cfg: bch.BenchRunCfg) -> bch.BenchCfg:
    #             return bch.BenchCfg()

    #     bench_runner = bch.BenchRunner(publisher=Mock())
    #     bench_fn1 = MockBenchable()
    #     bench_fn2 = MockBenchable()
    #     bench_runner.add_run(bench_fn1)
    #     bench_runner.add_run(bench_fn2)
    #     results = bench_runner.run(publish=True)
    #     self.assertEqual(len(results), 10)
    #     self.assertEqual(bench_runner.publisher.call_count, 10)
    #     self.assertEqual(bench_runner.publisher.call_args_list[0][0][0], results[0])
    #     self.assertEqual(bench_runner.publisher.call_args_list[1][0][0], results[1])
    #     self.assertEqual(bench_runner.publisher.call_args_list[2][0][0], results[2])
    #     self.assertEqual(bench_runner.publisher.call_args_list[3][0][0], results[3])
    #     self.assertEqual(bench_runner.publisher.call_args_list[4][0][0], results[4])
    #     self.assertEqual(bench_runner.publisher.call_args_list[5][0][0], results[5])
    #     self.assertEqual(bench_runner.publisher.call_args_list[6][0][0], results[6])
    #     self.assertEqual(bench_runner.publisher.call_args_list[7][0][0], results[7])
    #     self.assertEqual(bench_runner.publisher.call_args_list[8][0][0], results[8])
    #     self.assertEqual(bench_runner.publisher.call_args_list[9][0][0], results[9])

    # Tests that bch.BenchRunner can handle empty list of Benchable functions

    # Tests that bch.BenchRunner can handle empty list of Benchable functions
    # def test_benchrunner_handle_empty_list(self):

    #     def benchable(run_cfg:bch.BenchRunCfg)->bch.BenchCfg:
    #         bench = bch.Bench("sbc",SimpleBenchClass(),run_cfg=run_cfg)
    #         return bench.plot_sweep("sweep1")

    #     bench_runner = bch.BenchRunner()
    #     bench_runner.add_run(benchable)

    #     results = bench_runner.run(run_cfg=bch.BenchRunCfg(run_tag="1"))

    #     self.assertEqual(results[0].bench_cfg.run_tag, "1")

    # Tests that bch.BenchRunner can handle empty list of Benchable functions



import unittest
import bencher as bch


class TestClassEnum(unittest.TestCase):
    def test_basic(self):
        instance1 = bch.ExampleEnum.to_class(bch.ExampleEnum.Class1)
        self.assertEqual(instance1.classname, "class1")
        instance2 = bch.ExampleEnum.to_class(bch.ExampleEnum.Class2)
        self.assertEqual(instance2.classname, "class2")



# Generated by CodiumAI
from bencher.results.composable_container.composable_container_base import (
    ComposeType,
    ComposableContainerBase,
)


import pytest


class TestFlip:

    # flipping ComposeType.right returns ComposeType.down
    def test_flip_right_returns_down(self):
        assert ComposeType.right.flip() == ComposeType.down

    # flipping ComposeType.down returns ComposeType.right
    def test_flip_down_returns_right(self):
        assert ComposeType.down.flip() == ComposeType.right

    # flipping an invalid ComposeType raises RuntimeError
    def test_flip_invalid_raises_runtime_error(self):
        with pytest.raises(RuntimeError):
            invalid_type = ComposeType.sequence
            invalid_type.flip()

    # method should handle large number of flip calls without performance degradation
    def test_flip_large_number_of_calls(self):
        for _ in range(1000000):
            assert ComposeType.right.flip() == ComposeType.down
            assert ComposeType.down.flip() == ComposeType.right

    # method should maintain immutability of the original ComposeType instance
    def test_flip_maintains_immutability(self):
        original = ComposeType.right
        flipped = original.flip()
        assert original == ComposeType.right
        assert flipped == ComposeType.down


# Generated by CodiumAI
class TestComposableContainerBase:

    # append method adds an object to the container
    def test_append_adds_object(self):
        container = ComposableContainerBase()
        obj = "test_object"
        container.append(obj)
        assert obj in container.container

    # render method returns the current state of the container
    def test_render_returns_container_state(self):
        container = ComposableContainerBase()
        obj = "test_object"
        container.append(obj)
        assert container.render() == [obj]

    # Ensure label_formatter formats int values correctly with spaces included
    def test_label_formatter_formats_int_with_spaces(self):
        formatted_label = ComposableContainerBase.label_formatter("var", 12345)
        assert formatted_label == "var=  12345."

    # label_formatter handles None values for var_name and var_value
    def test_label_formatter_handles_none(self):
        formatted_label = ComposableContainerBase.label_formatter(None, None)
        assert formatted_label is None

    # append method handles appending None
    def test_append_handles_none(self):
        container = ComposableContainerBase()
        container.append(None)
        assert None in container.container

    # render method handles an empty container
    def test_render_empty_container(self):
        container = ComposableContainerBase()
        assert container.render() == []



import unittest
import bencher as bch
from bencher.example.example_floats2D import example_floats2D
from bencher.example.example_pareto import example_pareto
from bencher.example.example_simple_cat import example_1D_cat
from bencher.example.example_simple_float import example_1D_float
from bencher.example.example_simple_float2d import example_2D_float
from bencher.example.example_consts import example_2D_float_const
from bencher.example.example_float_cat import run_example_float_cat
from bencher.example.example_time_event import run_example_time_event
from bencher.example.example_float3D import example_floats3D

from bencher.example.example_custom_sweep import example_custom_sweep
from bencher.example.example_custom_sweep2 import example_custom_sweep2
from bencher.example.example_levels2 import example_levels2
from bencher.example.example_workflow import example_floats2D_workflow, example_floats3D_workflow
from bencher.example.example_holosweep import example_holosweep
from bencher.example.example_holosweep_tap import example_holosweep_tap

from bencher.example.optuna.example_optuna import optuna_rastrigin
from bencher.example.example_sample_cache import example_sample_cache
from bencher.example.example_strings import example_strings
from bencher.example.example_image import example_image, example_image_vid
from bencher.example.example_video import example_video
from bencher.example.example_filepath import example_filepath
from bencher.example.meta.example_meta import example_meta
from bencher.example.example_docs import example_docs

import os

# shelved
# from bencher.example.shelved.example_float2D_scatter import example_floats2D_scatter
# from bencher.example.shelved.example_float3D_cone import example_cone


class TestBenchExamples(unittest.TestCase):
    """The purpose of this test class is to run the example problems to make sure they don't crash.  The bencher logic is tested in the other test files test_bencher.py and test_vars.py"""

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.generate_all = False

    def create_run_cfg(self) -> bch.BenchRunCfg:
        cfg = bch.BenchRunCfg()
        if not self.generate_all:
            cfg.repeats = 2  # low number of repeats to reduce test time, but also test averaging and variance code
            cfg.level = 2  # reduce size of param sweep so tests are faster
        cfg.clear_cache = True
        return cfg

    def examples_asserts(self, example_result, save=False) -> None:
        self.assertIsNotNone(example_result)
        if save or self.generate_all:
            path = example_result.report.save_index("cachedir")
            self.assertTrue(os.path.exists(path))

    def test_publish_docs(self):
        report = example_docs(run_cfg=self.create_run_cfg())
        index = report.save_index(filename="index_tmp.html")
        index.unlink()  # remove tmp index

    # def test_example_categorical(self) -> None:
    #     self.examples_asserts(example_categorical(self.create_run_cfg()))

    # def test_example_floats(self) -> None:
    #     self.examples_asserts(example_floats(self.create_run_cfg()))

    def test_example_simple_cat(self) -> None:
        self.examples_asserts(example_1D_cat(self.create_run_cfg()))

    def test_example_floats2D(self) -> None:
        self.examples_asserts(example_floats2D(self.create_run_cfg()))

    def test_example_pareto(self) -> None:
        self.examples_asserts(example_pareto(self.create_run_cfg()))

    def test_example_simple_float(self) -> None:
        self.examples_asserts(example_1D_float(self.create_run_cfg()))

    def test_example_simple_float1D(self) -> None:
        self.examples_asserts(example_1D_float(self.create_run_cfg()))

    def test_example_simple_float2D(self) -> None:
        self.examples_asserts(example_2D_float(self.create_run_cfg()))

    def test_example_2D_float_const(self) -> None:
        self.examples_asserts(example_2D_float_const(self.create_run_cfg()))

    def test_example_float_cat(self) -> None:
        self.examples_asserts(run_example_float_cat(self.create_run_cfg()))

    def test_example_time_event(self) -> None:
        self.examples_asserts(run_example_time_event(self.create_run_cfg()))

    def test_example_float3D(self) -> None:
        self.examples_asserts(example_floats3D(self.create_run_cfg()))

    def test_example_custom_sweep(self) -> None:
        self.examples_asserts(example_custom_sweep(self.create_run_cfg()))

    def test_example_custom2(self) -> None:
        self.examples_asserts(example_custom_sweep2(self.create_run_cfg()))

    def test_example_level2(self) -> None:
        self.examples_asserts(example_levels2(self.create_run_cfg()))

    def test_example_floats2D_workflow(self) -> None:
        self.examples_asserts(example_floats2D_workflow(self.create_run_cfg()))

    def test_example_floats3D_workflow(self) -> None:
        self.examples_asserts(example_floats3D_workflow(self.create_run_cfg()))

    def test_holosweep(self) -> None:
        self.examples_asserts(example_holosweep(self.create_run_cfg()))

    def test_holosweep_tap(self) -> None:
        self.examples_asserts(example_holosweep_tap(self.create_run_cfg()))

    def test_optuna_rastrigin(self) -> None:
        self.examples_asserts(optuna_rastrigin(self.create_run_cfg()))

    def test_example_sample_cache(self) -> None:
        self.examples_asserts(example_sample_cache(self.create_run_cfg(), trigger_crash=False))

    def test_example_strings(self) -> None:
        self.examples_asserts(example_strings(self.create_run_cfg()))

    def test_example_image(self) -> None:
        self.examples_asserts(example_image(self.create_run_cfg()))
        self.examples_asserts(example_image_vid(self.create_run_cfg()))

    def test_example_video(self) -> None:
        self.examples_asserts(example_video(self.create_run_cfg()))

    def test_example_filepath(self) -> None:
        self.examples_asserts(example_filepath(self.create_run_cfg()))

    def test_example_meta(self) -> None:
        self.examples_asserts(example_meta(self.create_run_cfg()))

    # def test_example_meta_scatter(self) -> None:
    # self.examples_asserts(example_meta_scatter(self.create_run_cfg()))

    # shelved
    # def test_example_cone(self) -> None:
    #     self.examples_asserts(example_cone(self.create_run_cfg()))
    # def test_float2D_scatter(self) -> None:
    #     self.examples_asserts(example_floats2D_scatter(self.create_run_cfg()))



import unittest
import bencher as bch
import random
from bencher.job import JobFunctionCache

from hypothesis import given, strategies as st, settings


class CachedParamExample(bch.CachedParams):
    var1 = bch.FloatSweep(default=0, bounds=[0, 10])
    var2 = bch.IntSweep(default=10, bounds=[0, 10])

    result = bch.ResultVar()

    def __call__(self, **kwargs):
        self.update_params_from_kwargs(**kwargs)
        self.result = self.var1 + self.var2 + random.uniform(0, 1)
        return self.get_results_values_as_dict()


class TestJob(unittest.TestCase):
    @settings(deadline=500)
    @given(st.sampled_from([bch.Executors.SERIAL, bch.Executors.MULTIPROCESSING]))
    def test_basic(self, executor):
        cp = CachedParamExample()  # clears cache by default

        jc = JobFunctionCache(cp.__call__, executor=executor, cache_name="test_cache")
        jc.clear_cache()

        res1 = jc.call(var1=1).result()
        res2 = jc.call(var1=1).result()
        res3 = jc.call(var1=2).result()
        res4 = jc.call(var2=2).result()

        # will only be equal if cache is used because of the randomness
        self.assertEqual(res1["result"], res2["result"])
        self.assertNotEqual(res1["result"], res3["result"], f"{res1}")
        self.assertNotEqual(res1["result"], res4["result"], f"{res1}")

        # create new class, make sure it has the same results
        cp2 = CachedParamExample()
        jc2 = JobFunctionCache(cp2.__call__, executor=executor, cache_name="test_cache")
        res1cp2 = jc2.call(var1=1).result()
        self.assertEqual(res1["result"], res1cp2["result"])

        # create cache with a different name and check it does not have the same results
        cp3 = CachedParamExample()
        jc3 = JobFunctionCache(cp3.__call__, executor=executor, cache_name="test_cache2")
        res1cp3 = jc3.call(var1=1).result()
        self.assertNotEqual(res1["result"], res1cp3["result"])

    @settings(deadline=500)
    @given(st.sampled_from([bch.Executors.SERIAL, bch.Executors.MULTIPROCESSING]))
    def test_overwrite(self, executor):
        cp = CachedParamExample()  # clears cache by default

        jc = JobFunctionCache(cp.__call__, executor=executor, cache_name="test_cache1")
        jc.clear_cache()

        res1 = jc.call(var1=1).result()

        self.assertEqual(jc.worker_wrapper_call_count, 1)
        self.assertEqual(jc.worker_cache_call_count, 0)
        self.assertEqual(jc.worker_fn_call_count, 1)

        jc.clear_call_counts()
        res2 = jc.call(var1=1).result()

        self.assertEqual(jc.worker_wrapper_call_count, 1)
        self.assertEqual(jc.worker_cache_call_count, 1)
        self.assertEqual(jc.worker_fn_call_count, 0)

        self.assertEqual(res1["result"], res2["result"])

        jc.clear_call_counts()
        jc.overwrite = True
        res3 = jc.call(var1=1).result()
        self.assertEqual(jc.worker_wrapper_call_count, 1)
        self.assertEqual(jc.worker_cache_call_count, 0)
        self.assertEqual(jc.worker_fn_call_count, 1)

        self.assertNotEqual(res1["result"], res3["result"], f"{res1}")

    @settings(deadline=1000)
    @given(st.sampled_from([bch.Executors.SERIAL, bch.Executors.MULTIPROCESSING]))
    def test_bench_runner_parallel(self, executor):
        run_cfg = bch.BenchRunCfg()
        run_cfg.overwrite_sample_cache = True
        run_cfg.executor = executor
        bench_run = bch.BenchRunner("test_bench_runner", run_cfg=run_cfg)

        bench_run.add_bench(CachedParamExample())

        bench_run.run(level=2)


if __name__ == "__main__":
    TestJob().test_bench_runner_parallel(True).report.show()



# Generated by CodiumAI

import pytest
from bencher import p


class TestP:

    # returns correct dictionary with all parameters provided
    def test_returns_correct_dict_all_params(self):
        result = p(name="param1", values=[1, 2, 3], samples=10, max_level=5)
        assert result == {"name": "param1", "values": [1, 2, 3], "samples": 10, "max_level": 5}

    # returns correct dictionary with only name provided
    def test_returns_correct_dict_name_only(self):
        result = p(name="param1")
        assert result == {"name": "param1", "values": None, "samples": None, "max_level": None}

    # returns correct dictionary with name and values provided
    def test_returns_correct_dict_name_and_values(self):
        result = p(name="param1", values=[1, 2, 3])
        assert result == {"name": "param1", "values": [1, 2, 3], "samples": None, "max_level": None}

    # raises ValueError when max_level is 0
    def test_raises_value_error_max_level_zero(self):
        with pytest.raises(ValueError, match="max_level must be greater than 0"):
            p(name="param1", max_level=0)

    # raises ValueError when max_level is negative
    def test_raises_value_error_max_level_negative(self):
        with pytest.raises(ValueError, match="max_level must be greater than 0"):
            p(name="param1", max_level=-1)

    # raises ValueError when samples is 0
    def test_raises_value_error_samples_zero(self):
        with pytest.raises(ValueError, match="samples must be greater than 0"):
            p(name="param1", samples=0)



# Generated by CodiumAI
from bencher.bench_report import BenchReport
import unittest
import panel as pn


class TestBenchReport(unittest.TestCase):
    # Tests that the BenchReport instance is initialized with the provided bench_name parameter
    def test_init_with_bench_name(self):
        bench_name = "Test Bench"
        bench_report = BenchReport(bench_name)
        self.assertEqual(bench_report.bench_name, bench_name)

    # Tests that a Markdown pane with a custom name is appended to the BenchReport instance
    def test_append_markdown_with_custom_name(self):
        bench_report = BenchReport()
        markdown = "# Test Markdown"
        name = "Custom Markdown"
        md_pane = bench_report.append_markdown(markdown, name=name)
        self.assertEqual(md_pane.name, name)
        self.assertEqual(md_pane.object, markdown)

    # Tests that a panel with a custom name is appended to the BenchReport instance
    def test_append_panel_with_custom_name(self):
        bench_report = BenchReport()
        panel = pn.panel("Test Panel")
        name = "Custom Panel"
        bench_report.append(panel, name=name)
        self.assertEqual(bench_report.pane[-1].name, name)
        self.assertEqual(bench_report.pane[-1].objects[0], panel)

    def test_append__col_panel_with_custom_name(self):
        bench_report = BenchReport()
        panel = pn.panel("Test Panel")
        name = "Custom Panel"
        bench_report.append_col(panel, name=name)
        self.assertEqual(bench_report.pane[-1].name, name)
        self.assertEqual(bench_report.pane[-1].objects[0], panel)

    def test_append_column(self):
        bench_report = BenchReport()
        panel = pn.panel("Test Panel")
        bench_report.append_col(panel)
        self.assertEqual(bench_report.pane[-1].name, panel.name)

    def test_append_tab(self):
        bench_report = BenchReport()
        panel = pn.panel("Test Panel")
        bench_report.append_tab(panel)
        self.assertEqual(bench_report.pane[-1].name, panel.name)

    def test_append_title(self):
        bench_report = BenchReport()
        bench_report.append_title("Title1")
        self.assertEqual(bench_report.pane[-1].name, "Title1")
